﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using System.Reflection;

namespace HeuristicLab.Parameters {
  [StorableClass]
  public class ParameterSecondLevelActualNameBinding : IParameterBinding {
    [Storable]
    private IValueParameter source;
    public IValueParameter Source {
      get { return source; }
    }
    [Storable]
    private IValueParameter destination;
    public IValueParameter Destination {
      get { return destination; }
    }
    [Storable]
    private string sourcePropertyName;
    public string SourcePropertyName {
      get { return sourcePropertyName; }
    }
    [Storable]
    private string destinationPropertyName;
    public string DestinationPropertyName {
      get { return destinationPropertyName; }
    }

    [StorableConstructor]
    protected ParameterSecondLevelActualNameBinding(bool deserializing) { }
    protected ParameterSecondLevelActualNameBinding(ParameterSecondLevelActualNameBinding original, Cloner cloner) {
      this.source = cloner.Clone(source);
      this.destination = cloner.Clone(destination);
      RegisterEventHandlers();
    }
    public ParameterSecondLevelActualNameBinding(IValueParameter source, IValueParameter destination, string sourcePropertyName, string destinationPropertyName) {
      if (source == null) throw new ArgumentNullException("source");
      if (destination == null) throw new ArgumentNullException("destination");
      this.source = source;
      this.destination = destination;
      this.sourcePropertyName = sourcePropertyName;
      this.destinationPropertyName = destinationPropertyName;
      RegisterEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterSecondLevelActualNameBinding(this, cloner);
    }

    public void Bind() {
      RegisterEventHandlers();
    }

    public void Unbind() {
      DeregisterEventHandlers();
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEventHandlers();
    }

    private void RegisterEventHandlers() {
      source.ValueChanged += new EventHandler(source_ValueChanged);
      ILookupParameter sourceParameter = GetNestedParameter(source, sourcePropertyName);
      if (sourceParameter != null)
        sourceParameter.ActualNameChanged += new EventHandler(sourceParameter_ActualNameChanged);
    }

    private void DeregisterEventHandlers() {
      source.ValueChanged -= new EventHandler(source_ValueChanged);

      ILookupParameter sourceParameter = GetNestedParameter(source, sourcePropertyName);
      if (sourceParameter != null)
        sourceParameter.ActualNameChanged -= new EventHandler(sourceParameter_ActualNameChanged);
    }

    private ILookupParameter GetNestedParameter(IValueParameter param, string name) {
      if (param.Value == null) return null;
      PropertyInfo property = param.Value.GetType().GetProperty(name, BindingFlags.Instance | BindingFlags.NonPublic);
      if (property == null) return null;
      if (property.GetGetMethod() == null) throw new InvalidOperationException("Property " + name + " of type " + param.Value.GetType().FullName + " does not have a Get method.");
      ILookupParameter nestedParam = (property.GetGetMethod().Invoke(param.Value, null) as ILookupParameter);
      return nestedParam;
    }

    private void source_ValueChanged(object sender, EventArgs e) {
      ILookupParameter sourceParameter = GetNestedParameter(source, sourcePropertyName);
      if (sourceParameter != null) {
        sourceParameter.ActualNameChanged += new EventHandler(sourceParameter_ActualNameChanged);
        DoAction();
      }
    }

    private void sourceParameter_ActualNameChanged(object sender, EventArgs e) {
      DoAction();
    }

    private void DoAction() {
      ILookupParameter sourceParameter = GetNestedParameter(source, sourcePropertyName);
      ILookupParameter destinationParameter = GetNestedParameter(destination, destinationPropertyName);
      if (destinationParameter != null)
        destinationParameter.ActualName = sourceParameter.ActualName;
    }
  }
}
