﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;

namespace HeuristicLab.Parameters {
  [StorableClass]
  public class ParameterValueBinding : IParameterBinding {
    [Storable]
    private IValueParameter recipient;
    public IValueParameter Recipient {
      get { return recipient; }
    }
    [Storable]
    private IValueParameter donor;
    public IValueParameter Donor {
      get { return donor; }
    }

    [StorableConstructor]
    protected ParameterValueBinding(bool deserializing) { }
    protected ParameterValueBinding(ParameterValueBinding original, Cloner cloner) {
      this.donor = cloner.Clone(donor);
      this.recipient = cloner.Clone(recipient);
      RegisterEventHandlers();
    }
    public ParameterValueBinding(IValueParameter recipient, IValueParameter donor) {
      if (recipient == null) throw new ArgumentNullException("destination");
      if (donor == null) throw new ArgumentNullException("source");
      this.recipient = recipient;
      this.donor = donor;
      RegisterEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParameterValueBinding(this, cloner);
    }

    public void Bind() {
      RegisterEventHandlers();
    }

    public void Unbind() {
      DeregisterEventHandlers();
    }

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
      RegisterEventHandlers();
    }

    private void RegisterEventHandlers() {
      donor.ValueChanged += new EventHandler(source_ValueChanged);
    }

    private void DeregisterEventHandlers() {
      donor.ValueChanged -= new EventHandler(source_ValueChanged);
    }
    

    private void source_ValueChanged(object sender, EventArgs e) {
      recipient.Value = (IItem)donor.Value.Clone();
    }
  }
}
